@@ -14,6 +14,7 @@ function (formula, locations, data)
     if (length(Y) == 0) 
         stop("no response variable present in formula")
     X = model.matrix(Terms, m)
+	has.intercept = attr(Terms, "intercept")
     if (class(locations) != "formula") 
         stop("locations argument should be a formula, such as ~x+y")
     m = match.call(expand = FALSE)
@@ -27,5 +28,6 @@ function (formula, locations, data)
     if ((yvar = attr(Terms, "response")) > 0) 
         stop("no response allowed in locations formula")
     locations = model.matrix(Terms, m)
-    list(y = Y, locations = locations, X = X, call = call)
+    list(y = Y, locations = locations, X = X, call = call, has.intercept =
+		has.intercept)
 }
