@@ -13,7 +13,7 @@ function(object, maxdist, n=200, min=1.0e-6*maxdist, ids=c(0,0),
 	pars = c(min,maxdist,n,dir)
 	load.variogram.model(object, ids) # loads object into gstat 
 	ret = .Call("gstat_variogram_values", as.integer(ids),
-		as.numeric(pars))
-	.Call("gstat_exit", 0);
+		as.numeric(pars), PACKAGE = "gstat")
+	.Call("gstat_exit", 0, PACKAGE = "gstat");
 	data.frame(dist=ret[[1]], gamma=ret[[2]])
 }
